/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy 
 * http://www.e-taxonomy.eu
 * 
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.preference;

import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.UUID;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.swt.widgets.Shell;

import eu.etaxonomy.cdm.api.facade.DerivedUnitFacadeConfigurator;
import eu.etaxonomy.cdm.api.service.IFeatureTreeService;
import eu.etaxonomy.cdm.api.service.ITermService;
import eu.etaxonomy.cdm.api.service.config.FindTaxaAndNamesConfiguratorImpl;
import eu.etaxonomy.cdm.api.service.config.IFindTaxaAndNamesConfigurator;
import eu.etaxonomy.cdm.common.CdmUtils;
import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.common.ICdmBase;
import eu.etaxonomy.cdm.model.common.IDefinedTerm;
import eu.etaxonomy.cdm.model.common.Language;
import eu.etaxonomy.cdm.model.common.MarkerType;
import eu.etaxonomy.cdm.model.common.TermBase;
import eu.etaxonomy.cdm.model.description.FeatureTree;
import eu.etaxonomy.cdm.model.name.NomenclaturalCode;
import eu.etaxonomy.cdm.strategy.match.DefaultMatchStrategy;
import eu.etaxonomy.cdm.strategy.match.IMatchStrategy;
import eu.etaxonomy.cdm.strategy.match.MatchException;
import eu.etaxonomy.cdm.strategy.match.MatchMode;
import eu.etaxonomy.taxeditor.model.NomenclaturalCodeHelper;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.store.StoreUtil;
import eu.etaxonomy.taxeditor.store.internal.TaxeditorStorePlugin;

/**
 * <p>
 * PreferencesUtil class.
 * </p>
 * 
 * @author p.ciardelli
 * @author n.hoffmann
 * @created 05.12.2008
 * @version 1.0
 */
public class PreferencesUtil implements IPreferenceKeys {

	/**
	 * 
	 */
	public static final String PREFERRED_TERMS_CHANGE = "preferred_terms";

	/**
	 * <p>
	 * getPreferenceStore
	 * </p>
	 * 
	 * @return a {@link org.eclipse.jface.preference.IPreferenceStore} object.
	 */
	public static IPreferenceStore getPreferenceStore() {
		return TaxeditorStorePlugin.getDefault().getPreferenceStore();
	}

	/**
	 * <p>
	 * setPreferredNomenclaturalCode
	 * </p>
	 * 
	 * @param preferredCode
	 *            a {@link eu.etaxonomy.cdm.model.name.NomenclaturalCode}
	 *            object.
	 */
	public static void setPreferredNomenclaturalCode(
			NomenclaturalCode preferredCode) {
		getPreferenceStore().setValue(PREFERRED_NOMENCLATURAL_CODE_KEY,
				getPreferenceKey(preferredCode));
	}

	/**
	 * <p>
	 * getPreferredNomenclaturalCode
	 * </p>
	 * 
	 * @return a {@link eu.etaxonomy.cdm.model.name.NomenclaturalCode} object.
	 */
	public static NomenclaturalCode getPreferredNomenclaturalCode() {

		for (NomenclaturalCode code : NomenclaturalCodeHelper.getAllCodes()) {
			String preferredCode = getPreferenceStore().getString(
					PREFERRED_NOMENCLATURAL_CODE_KEY);
			if (getPreferenceKey(code).equals(preferredCode)) {
				return code;
			}
		}
		return null;
	}

	/**
	 * Get the match strategy for the given class that was stored in preferences
	 * or the default strategy if it was not stored in preferences
	 * 
	 * @param clazz
	 *            a {@link java.lang.Class} object.
	 * @return a {@link eu.etaxonomy.cdm.strategy.match.IMatchStrategy} object.
	 */
	public static IMatchStrategy getMatchStrategy(Class clazz) {
		String className = clazz.getName();
		if (getPreferenceStore().getBoolean(MATCH_STRATEGY_PREFIX + className)) {
			IMatchStrategy matchStrategy = getDefaultMatchStrategy(clazz);

			for (String fieldName : matchStrategy.getMatchFieldPropertyNames()) {
				String matchModeName = getPreferenceStore().getString(
						getMatchStrategyFieldName(className, fieldName));
				MatchMode matchMode = MatchMode.valueOf(matchModeName);
				try {
					matchStrategy.setMatchMode(fieldName, matchMode);
				} catch (MatchException e) {
					StoreUtil.error(PreferencesUtil.class, e);
					throw new RuntimeException(e);
				}
			}

			return matchStrategy;
		}
		return getDefaultMatchStrategy(clazz);
	}

	/**
	 * Stores a matchStrategy into the preference store.
	 * 
	 * @param matchStrategy
	 *            a {@link eu.etaxonomy.cdm.strategy.match.IMatchStrategy}
	 *            object.
	 */
	public static void setMatchStrategy(IMatchStrategy matchStrategy) {
		String className = matchStrategy.getMatchClass().getName();
		getPreferenceStore().setValue(MATCH_STRATEGY_PREFIX + className, true);

		Set<String> matchFields = matchStrategy.getMatchFieldPropertyNames();

		for (String fieldName : matchFields) {
			getPreferenceStore().setValue(
					getMatchStrategyFieldName(className, fieldName),
					matchStrategy.getMatchMode(fieldName).name());
		}
	}

	/**
	 * Helper method to create the preference property for a match field.
	 * 
	 * @param className
	 * @param fieldName
	 * @return
	 */
	private static String getMatchStrategyFieldName(String className,
			String fieldName) {
		return MATCH_STRATEGY_PREFIX + className + "." + fieldName;
	}

	/**
	 * Returns the default match strategy for a given class.
	 * 
	 * @param clazz
	 *            a {@link java.lang.Class} object.
	 * @return a {@link eu.etaxonomy.cdm.strategy.match.IMatchStrategy} object.
	 */
	public static IMatchStrategy getDefaultMatchStrategy(Class clazz) {
		return DefaultMatchStrategy.NewInstance(clazz);
	}

	/**
	 * <p>
	 * getDateFormatPattern
	 * </p>
	 * 
	 * @return a {@link java.lang.String} object.
	 */
	public static String getDateFormatPattern() {
		// TODO make this configurable in properties
		String pattern = "Y-M-d H:m";
		return pattern;
	}

	/**
	 * <p>
	 * addTermToPreferredTerms
	 * </p>
	 * 
	 * @param term
	 *            a T object.
	 * @param <T>
	 *            a T object.
	 */
	public static <T extends TermBase> void addTermToPreferredTerms(T term) {

		// VocabularyEnum vocabulary =
		// VocabularyEnum.getVocabularyEnum(term.getClass());
		//
		// getPreferenceStore().setValue(getPreferenceKey(term),
		// VocabularyStore.getTermVocabulary(vocabulary).getTerms().contains(term));
		//
		// firePreferencesChanged(term.getClass());
	}

	/**
	 * Construct a unique key using the CdmBase object's uuid
	 * 
	 * @param cdmBase
	 * @return
	 */
	private static String getPreferenceKey(ICdmBase cdmBase) {
		cdmBase = (ICdmBase) HibernateProxyHelper.deproxy(cdmBase);

		String key = cdmBase.getClass().getName().concat(".")
				.concat(cdmBase.getUuid().toString());
		if (key.contains("javassist")) {
			StoreUtil.info("proxy");
		}
		return key;
	}

	/**
	 * Construct a unique key using the CdmBase object's uuid
	 * 
	 * @param cdmBase
	 * @return
	 */
	public static String getPreferenceKey(IDefinedTerm definedTerm) {
		definedTerm = (IDefinedTerm) HibernateProxyHelper.deproxy(definedTerm);
		String key = definedTerm.getClass().getName().concat(".")
				.concat(definedTerm.getUuid().toString());
		if (key.contains("javassist")) {
			StoreUtil.warn(PreferencesUtil.class,
					"Trying to persist a preference based on a proxy class.");
		}
		return key;
	}

	/**
	 * Retrieves search preferences from the preference store
	 * 
	 * @return an {@link ITaxonServiceConfigurator} to pass to search methods
	 */
	public static IFindTaxaAndNamesConfigurator getSearchConfigurator() {
		IFindTaxaAndNamesConfigurator configurator = initializeSearchConfigurator();

		configurator.setDoTaxa(getPreferenceStore().getBoolean(
				TAXON_SERVICE_CONFIGURATOR_TAXA));
		configurator.setDoSynonyms(getPreferenceStore().getBoolean(
				TAXON_SERVICE_CONFIGURATOR_SYNONYMS));
		configurator.setDoNamesWithoutTaxa(getPreferenceStore().getBoolean(
				TAXON_SERVICE_CONFIGURATOR_NAMES));
		configurator.setDoTaxaByCommonNames(getPreferenceStore().getBoolean(
				TAXON_SERVICE_CONFIGURATOR_COMMON_NAMES));

		return configurator;
	}

	/**
	 * create new preferences, setting all search options to true
	 * 
	 * @return a
	 *         {@link eu.etaxonomy.cdm.api.service.config.ITaxonServiceConfigurator}
	 *         object.
	 */
	public static IFindTaxaAndNamesConfigurator initializeSearchConfigurator() {
		IFindTaxaAndNamesConfigurator configurator = new FindTaxaAndNamesConfiguratorImpl();

		configurator.setDoTaxa(true);
		configurator.setDoSynonyms(true);
		configurator.setDoNamesWithoutTaxa(true);
		configurator.setDoTaxaByCommonNames(true);

		configurator.setTaxonPropertyPath(Arrays.asList("$", "titleCache",
				"name", "name.$", "relationsFromThisTaxon.$"));

		configurator.setSynonymPropertyPath(Arrays.asList("$", "titleCache",
				"name", "name.$", "synonymRelations.relatedTo.*"));

		// DEFAULT VALUES
		// match mode is a simple like, actually all other match modes are kind
		// of bogus
		configurator
				.setMatchMode(eu.etaxonomy.cdm.persistence.query.MatchMode.ANYWHERE);
		// we set page number and size here as this should always be unlimited
		configurator.setPageNumber(0);
		// TODO currently limit results to 10000
		configurator.setPageSize(10000);

		return configurator;
	}

	/**
	 * Store search preferences
	 * 
	 * @param configurator
	 *            a
	 *            {@link eu.etaxonomy.cdm.api.service.config.ITaxonServiceConfigurator}
	 *            object.
	 */
	public static void setSearchConfigurator(
			IFindTaxaAndNamesConfigurator configurator) {
		getPreferenceStore().setValue(TAXON_SERVICE_CONFIGURATOR_TAXA,
				configurator.isDoTaxa());
		getPreferenceStore().setValue(TAXON_SERVICE_CONFIGURATOR_SYNONYMS,
				configurator.isDoSynonyms());
		getPreferenceStore().setValue(TAXON_SERVICE_CONFIGURATOR_NAMES,
				configurator.isDoNamesWithoutTaxa());
		getPreferenceStore().setValue(TAXON_SERVICE_CONFIGURATOR_COMMON_NAMES,
				configurator.isDoTaxaByCommonNames());
	}

	/**
	 * <p>
	 * firePreferencesChanged
	 * </p>
	 * 
	 * @param clazz
	 *            a {@link java.lang.Class} object.
	 */
	public static void firePreferencesChanged(Class clazz) {
		getPreferenceStore().firePropertyChangeEvent(PREFERRED_TERMS_CHANGE,
				null, clazz);
	}

	/**
	 * Set default values for preferences
	 */
	public static void setDefaults() {
		getPreferenceStore().setDefault(TAXON_SERVICE_CONFIGURATOR_TAXA, true);
		getPreferenceStore().setDefault(TAXON_SERVICE_CONFIGURATOR_SYNONYMS,
				true);
		getPreferenceStore().setDefault(EDIT_MAP_SERVICE_ACCES_POINT,
				"http://edit.br.fgov.be/edit_wp5/v1/areas.php");
		getPreferenceStore().setDefault(SHOULD_CONNECT_AT_STARTUP, true);
		getPreferenceStore().setDefault(OPENURL_ACCESS_POINT,
				"http://www.biodiversitylibrary.org/openurl");
		getPreferenceStore().setDefault(OPENURL_IMAGE_MAX_WIDTH, "1000");
		getPreferenceStore().setDefault(OPENURL_IMAGE_MAX_HEIGHT, "1000");
	}

	/**
	 * <p>
	 * checkNomenclaturalCode
	 * </p>
	 */
	public static void checkNomenclaturalCode() {
		// First time Editor is opened, no nomenclatural code has been set
		if (PreferencesUtil.getPreferredNomenclaturalCode() == null) {

			StoreUtil.info("No nomencatural code set.");

			Shell shell = StoreUtil.getShell();

			// Query user re: preferred nom. code
			Dialog dialog = new InitNomenclaturalCodePrefDialog(shell);
			dialog.open();

			// Short message confirming user's choice
			NomenclaturalCode code = PreferencesUtil
					.getPreferredNomenclaturalCode();
			MessageDialog
					.openInformation(
							shell,
							"Nomenclatural code set",
							"The following has been set as your preferred nomenclatural code:\n\n\t"
									+ NomenclaturalCodeHelper
											.getDescription(code)
									+ "\n\nYou can change the nomenclatural code at any time in the \"Preferences\" menu.");
		}
	}

	/**
	 * <p>
	 * getMapServiceAccessPoint
	 * </p>
	 * 
	 * @return a {@link java.lang.String} object.
	 */
	public static String getMapServiceAccessPoint() {
		return getPreferenceStore().getString(EDIT_MAP_SERVICE_ACCES_POINT);
	}

	/**
	 * <p>
	 * shouldConnectAtStartUp
	 * </p>
	 * 
	 * @return a boolean.
	 */
	public static boolean shouldConnectAtStartUp() {
		return getPreferenceStore().getBoolean(SHOULD_CONNECT_AT_STARTUP);
	}

	/**
	 * <p>
	 * getDefaultFeatureTreeForTextualDescription
	 * </p>
	 * 
	 * @return a {@link eu.etaxonomy.cdm.model.description.FeatureTree} object.
	 */
	public static FeatureTree getDefaultFeatureTreeForTextualDescription() {
		String uuidString = getPreferenceStore().getString(
				FEATURE_TREE_DEFAULT_TEXT);
		return CdmUtils.isEmpty(uuidString) ? null : CdmStore.getService(
				IFeatureTreeService.class).load(UUID.fromString(uuidString));
	}

	/**
	 * <p>
	 * getDefaultFeatureTreeForStructuredDescription
	 * </p>
	 * 
	 * @return a {@link eu.etaxonomy.cdm.model.description.FeatureTree} object.
	 */
	public static FeatureTree getDefaultFeatureTreeForStructuredDescription() {
		String uuidString = getPreferenceStore().getString(
				FEATURE_TREE_DEFAULT_STRUCTURE);
		return CdmUtils.isEmpty(uuidString) ? null : CdmStore.getService(
				IFeatureTreeService.class).load(UUID.fromString(uuidString));
	}

	/**
	 * <p>
	 * setSortRanksHierarchichally
	 * </p>
	 * 
	 * @param selection
	 *            a boolean.
	 */
	public static void setSortRanksHierarchichally(boolean selection) {
		getPreferenceStore().setValue(SORT_RANKS_HIERARCHICHALLY, selection);
	}

	/**
	 * <p>
	 * getSortRanksHierarchichally
	 * </p>
	 * 
	 * @return a boolean.
	 */
	public static boolean getSortRanksHierarchichally() {
		return getPreferenceStore().getBoolean(SORT_RANKS_HIERARCHICHALLY);
	}

	public static boolean isMultilanguageTextEditingCapability() {
		return getPreferenceStore().getBoolean(
				MULTILANGUAGE_TEXT_EDITING_CAPABILITY);
	}

	public static Language getGlobalLanguage() {
		String languageUuidString = getPreferenceStore().getString(
				GLOBAL_LANGUAGE_UUID);

		if (CdmUtils.isEmpty(languageUuidString)) {
			return Language.DEFAULT();
		}

		UUID languageUuid = UUID.fromString(languageUuidString);
		return (Language) CdmStore.getService(ITermService.class).load(
				languageUuid);
	}

	public static void setGlobalLanguage(Language language) {
		getPreferenceStore().setValue(GLOBAL_LANGUAGE_UUID,
				language.getUuid().toString());
		CdmStore.setDefaultLanguage(language);
	}

	/**
	 * @return
	 */
	public static Map<MarkerType, Boolean> getEditMarkerTypePreferences() {
		List<MarkerType> markerTypes = CdmStore.getTermManager()
				.getPreferredTerms(MarkerType.class);

		Map<MarkerType, Boolean> result = new HashMap<MarkerType, Boolean>();

		for (MarkerType markerType : markerTypes) {
			String name = getMarkerTypeEditingPreferenceKey(markerType);
			Boolean value = getPreferenceStore().getBoolean(name);

			result.put(markerType, value);
		}

		return result;
	}

	/**
	 * @param markerTypeEditingMap
	 */
	public static void setEditMarkerTypePreferences(
			Map<MarkerType, Boolean> markerTypeEditingMap) {
		for (MarkerType markerType : markerTypeEditingMap.keySet()) {
			String name = getMarkerTypeEditingPreferenceKey(markerType);
			getPreferenceStore().setValue(name,
					markerTypeEditingMap.get(markerType));
		}

	}

	private static String getMarkerTypeEditingPreferenceKey(
			MarkerType markerType) {
		markerType = (MarkerType) HibernateProxyHelper.deproxy(markerType);
		return markerType.getClass().getName() + EDIT_MARKER_TYPE_PREFIX;
	}

	/**
	 * <p>
	 * setEditMarkerTypePreference
	 * </p>
	 * 
	 * @param input
	 *            a {@link org.eclipse.ui.IEditorInput} object.
	 * @param markerType
	 *            a {@link eu.etaxonomy.cdm.model.common.MarkerType} object.
	 * @param edit
	 *            a boolean.
	 */
	public static void setEditMarkerTypePreference(MarkerType markerType,
			boolean edit) {
		getPreferenceStore().setValue(
				getMarkerTypeEditingPreferenceKey(markerType), edit);
	}

	/**
	 * @return
	 */
	public static DerivedUnitFacadeConfigurator getDerivedUnitConfigurator() {
		DerivedUnitFacadeConfigurator configurator = DerivedUnitFacadeConfigurator
				.NewInstance();
		configurator.setMoveDerivedUnitMediaToGallery(true);
		configurator.setMoveFieldObjectMediaToGallery(true);
		return configurator;
	}

}
