// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.ui.dialog.selection;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.UUID;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Shell;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.api.service.IVocabularyService;
import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.model.common.TermVocabulary;
import eu.etaxonomy.cdm.model.common.UuidAndTitleCache;
import eu.etaxonomy.cdm.model.common.VocabularyEnum;
import eu.etaxonomy.cdm.model.location.NamedArea;
import eu.etaxonomy.taxeditor.newWizard.AbstractNewEntityWizard;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * <p>FilteredNamedAreaSelectionDialog class.</p>
 *
 * @author n.hoffmann
 * @created May 11, 2010
 * @version 1.0
 */
public class NamedAreaSelectionDialog extends
		AbstractFilteredCdmResourceSelectionDialog<NamedArea> {

	private class IncludeNamedAreaVocabulary extends Action {
		private TermVocabulary<NamedArea> vocabulary;

		/**
		 * Creates a new instance of the class.
		 */
		public IncludeNamedAreaVocabulary(TermVocabulary<NamedArea> vocabulary) {
			super(vocabulary.getTitleCache(), IAction.AS_CHECK_BOX);
			this.vocabulary = vocabulary;
		}
		
		public void run(){
			if(isChecked()){
				selectedVocabularies.add(vocabulary);
			}else{
				selectedVocabularies.remove(vocabulary);
			}
			
			initModel();
		}
	}
	
	private Collection<TermVocabulary<NamedArea>> selectedVocabularies;
	
	
	/**
	 * Creates a filtered selection dialog to select a named area.
	 *
	 * @param shell
	 * 				The shell for displaying this widget
	 * @param namedArea
	 * 				A namedArea that should be selected when the dialog opens
	 * @param conversation a {@link eu.etaxonomy.cdm.api.conversation.ConversationHolder} object.
	 * @return a {@link eu.etaxonomy.cdm.model.location.NamedArea} object.
	 */
	public static NamedArea select(Shell shell, ConversationHolder conversation, NamedArea namedArea) {
		NamedAreaSelectionDialog dialog = new NamedAreaSelectionDialog(shell, conversation,
				"Choose an area", false, namedArea);
		return getSelectionFromDialog(dialog);
	}
	
	/**
	 * <p>Constructor for FilteredNamedAreaSelectionDialog.</p>
	 *
	 * @param shell a {@link org.eclipse.swt.widgets.Shell} object.
	 * @param conversation a {@link eu.etaxonomy.cdm.api.conversation.ConversationHolder} object.
	 * @param title a {@link java.lang.String} object.
	 * @param multi a boolean.
	 * @param namedArea a {@link eu.etaxonomy.cdm.model.location.NamedArea} object.
	 */
	protected NamedAreaSelectionDialog(Shell shell, ConversationHolder conversation, String title, boolean multi, NamedArea namedArea) {
		super(shell, conversation, title, multi, NamedAreaSelectionDialog.class.getCanonicalName(), namedArea);		
	}
	
	/** {@inheritDoc} */
	@Override
	protected void fillViewMenu(IMenuManager menuManager) {
				
		super.fillViewMenu(menuManager);
				
		for(TermVocabulary<NamedArea> vocabulary : getVocabularies()){
			IncludeNamedAreaVocabulary action = new IncludeNamedAreaVocabulary(vocabulary);
			menuManager.add(action);
			action.setChecked(true);
		}
	}
	
	/** {@inheritDoc} */
	@Override
	protected NamedArea getPersistentObject(UUID uuid) {
		for(TermVocabulary<NamedArea> vocabulary : selectedVocabularies){
			for(Object object : vocabulary.getTerms()){
				CdmBase cdmBaseObject = (CdmBase) object;
				if(uuid.equals(cdmBaseObject.getUuid())){
					return (NamedArea) cdmBaseObject;
				}
			}
		}
		return null;
	}
	
	/** {@inheritDoc} */
	@Override
	protected void init() {
		selectedVocabularies = getVocabularies();
	}
	
	private List<TermVocabulary<NamedArea>> getVocabularies(){
		List<TermVocabulary<NamedArea>> vocabularies = CdmStore.getService(IVocabularyService.class).listByTermClass(NamedArea.class, null, null, null, null);
		vocabularies.add(CdmStore.getService(IVocabularyService.class).getVocabulary(VocabularyEnum.TdwgArea));
		vocabularies.add(CdmStore.getService(IVocabularyService.class).getVocabulary(VocabularyEnum.WaterbodyOrCountry));
		vocabularies.add(CdmStore.getService(IVocabularyService.class).getVocabulary(VocabularyEnum.Continent));
		return vocabularies;
	}

	/** {@inheritDoc} */
	@Override
	protected void initModel() {
		
		Set<NamedArea> terms = new HashSet<NamedArea>();
		for(TermVocabulary<NamedArea> vocabulary : selectedVocabularies){
			terms.addAll(vocabulary.getTermsOrderedByLabels(CdmStore.getDefaultLanguage()));
		}
		
		if(model == null){
			model = new ArrayList<UuidAndTitleCache<NamedArea>>();
		}
		model.clear();
		for(Object areaObject : terms){
			NamedArea area = (NamedArea) HibernateProxyHelper.deproxy(areaObject);
			UuidAndTitleCache<NamedArea> element = new UuidAndTitleCache<NamedArea>(NamedArea.class, area.getUuid(), getTitle(area)); 
			model.add(element);
		}
	}

	/** {@inheritDoc} */
	@Override
	protected Control createExtendedContentArea(Composite parent) {
		return null;
	}
	
	/** {@inheritDoc} */
	@Override
	protected String getTitle(NamedArea namedArea) {
		try {
			String result = NamedArea.labelWithLevel(namedArea, CdmStore.getDefaultLanguage());
			return result;
		} catch (Exception e) {
			//TODO still need to learn how errors are handled in the Tax Editor
			System.out.println("Error occurred when trying retrieve title for Named Area: " + namedArea.getUuid());
			return namedArea.getTitleCache();
		}
	}
	
	/** {@inheritDoc} */
	@Override
	protected AbstractNewEntityWizard getNewEntityWizard(String parameter) {
		return null;
	}

	/** {@inheritDoc} */
	@Override
	protected String getNewWizardLinkText() {
		return null;
	}
}
